#version 430 core

in vec3 tePosition;
in vec3 teNormal;
in vec3 tePatchDistance;
in vec4 shadow_coord;

uniform sampler2D tex_grass;
uniform sampler2D tex_dirt;
uniform sampler2D shadow_texture;
uniform mat3 normalMatrix;

out vec3 fragColor;

//Triplanar texturing http://http.developer.nvidia.com/GPUGems3/gpugems3_ch01.html

vec2 poissonDisk[4] = vec2[](
  vec2( -0.94201624, -0.39906216 ),
  vec2( 0.94558609, -0.76890725 ),
  vec2( -0.094184101, -0.92938870 ),
  vec2( 0.34495938, 0.29387760 )
);

void main()
{
	vec3 weights = abs(teNormal);
	weights = (weights - 0.2) * 7;  
	weights = max(weights, 0); 
	weights /= vec3(weights.x + weights.y + weights.z);


	float texSize = 1.0/70.0;
	vec3 col0 = vec3(texture(tex_dirt,tePosition.yz*texSize));
	vec3 col1 = vec3(texture(tex_grass,tePosition.xz*texSize));
	vec3 col2 = vec3(texture(tex_dirt,tePosition.xy*texSize));

	fragColor = col0.xyz * weights.xxx +  
                col1.xyz * weights.yyy +
                col2.xyz * weights.zzz;

    vec3 lightDir = vec3(0.3,1,0);
	float NdotL = max(dot(teNormal, lightDir), 0.0);

	//float bias = 0.003;
	float bias = 0.008*(1+1-NdotL);
	//bias = max(min(0,bias),0.01);
	float visibility = 1.0;
	
	for (int i=0;i<4;i++)
	{
		if (shadow_coord.z-bias - texture2D(shadow_texture, shadow_coord.xy + poissonDisk[i]/3000.0).r  > 0  )
		{
	    	visibility -= 0.2;
	    }
	}

/*	if (shadow_coord.z-bias - texture2D(shadow_texture, shadow_coord.xy).r  > 0  )
	{
    	visibility = 0.5;
    }*/

    fragColor = fragColor*(NdotL)*visibility;
}